@@ -36,6 +36,7 @@ class BaseModel:  # pylint: disable=too-few-public-methods,too-many-public-metho
     __schema__: tuple[Constraint, ...] = ()
     __validators__: dict[str, list[Validators]] = {}
     __hooks__: "dict[str, list[Callable[[Self], None] | str]]" = {}
+    __hidden__: tuple[str, ...] = ()
     _tbl: Table
     _primary: str | None
 
@@ -85,11 +86,6 @@ class BaseModel:  # pylint: disable=too-few-public-methods,too-many-public-metho
         except DatabaseExistsError:
             cls._tbl = db.table(cls.__table_name__.lower())
 
-    @classmethod
-    def where(cls, **kwargs):
-        """Basic select operation"""
-        return QueryBuilder(cls).where(**kwargs)
-
     @classmethod
     def _execute_hooks(cls, name: str, instance: Self):
         for hook in cls.__hooks__.get(name, ()):
@@ -103,6 +99,42 @@ class BaseModel:  # pylint: disable=too-few-public-methods,too-many-public-metho
         for validator in cls.__validators__.get(name, ()):
             validator.validate(instance)
 
+    @classmethod
+    def _register(cls, type_: str = 'hook', name: str = "", if_fail: str = ""):
+        """Register a hook/validator under a name"""
+        if type_ not in ("hook", "validator"):
+            raise ValueError("Which do you want?")
+        def function(func):
+            if name == '':
+                raise ValueError(f"{type_.title()} name needs to be declared.")
+            if type_ == 'hook':
+                if name not in VALID_HOOKS_NAME:
+                    raise ValueError("Name of a hook doesn't match with expected value")
+                cls.__hooks__.setdefault(name, [])
+                if cls.__hooks__[name]:
+                    cls.__hooks__[name] = [func]
+                else:
+                    cls.__hooks__[name].append(func)
+                return func
+
+            if not is_dataclass(cls):
+                raise TypeError("Dataclass is required for this class")
+
+            fields_ = tuple((field.name for field in fields(cls)))
+            if name not in fields_:
+                raise ValueError("Expected validator to has name as column field")
+
+            fail = if_fail or f"{name} fails certain validator"
+
+            cls.__validators__.setdefault(name, [])
+            validator = Validators(func, fail)
+            if cls.__validators__[name]:
+                cls.__validators__[name] = [validator]
+            else:
+                cls.__validators__[name].append(validator)
+            return func
+        return function
+
     @classmethod
     def create(cls, **kwargs):
         """Create data based on kwargs"""
@@ -215,9 +247,17 @@ class BaseModel:  # pylint: disable=too-few-public-methods,too-many-public-metho
     def to_dict(self):
         """Convert model instance to dictionary."""
         if is_dataclass(self):  # always true, though, just in case
-            return asdict(self)
+            instance = asdict(self)
+            return {k: v for k, v in instance if k not in self.__hidden__}
         return {}
 
+    def to_safe_instance(self):
+        """Wrap instance that complies with __hidden__."""
+        if is_dataclass(self):
+            instance = {k: (v if not k in self.__hidden__ else None) for k, v in asdict(self)}
+            return type(self)(**instance)
+        raise TypeError("This class must be a dataclass")
+
     def raw(self, query: str, params: list[Any] | tuple[Any, ...] | dict[str, Any]):
         """Raw SQL query"""
         return self._tbl._sql.execute(query, params)  # pylint: disable=protected-access
@@ -282,56 +322,29 @@ class BaseModel:  # pylint: disable=too-few-public-methods,too-many-public-metho
             f"with {self.__class__.__name__}"
         )
 
-    def hook(self, name: str):
+    @classmethod
+    def hook(cls, name: str):
         """Register a hook"""
-        return self.register('hook', name)
+        return cls._register('hook', name)
 
-    def validator(self, column_name: str, if_fail: str):
+    @classmethod
+    def validator(cls, column_name: str, if_fail: str):
         """Register a validator"""
-        return self.register("validator", column_name, if_fail)
+        return cls._register("validator", column_name, if_fail)
 
-    @overload
-    def register(self, type_: str = 'hook', name: str = ""):
-        """Register a hooks under a name"""
+    def get_table(self):
+        """Return table instance"""
+        return self._tbl
 
-    @overload
-    def register(self, type_: str = 'validator', name: str = "", if_fail: str = ""):
-        """Register a validator under a name"""
+    @classmethod
+    def where(cls, **kwargs):
+        """Basic select operation"""
+        return QueryBuilder(cls).where(**kwargs)
 
-    def register(self, type_: str = 'hook', name: str = "", if_fail: str = ""):
-        """Register a hook/validator under a name"""
-        if type_ not in ("hook", "validator"):
-            raise ValueError("Which do you want?")
-        def function(func):
-            if name == '':
-                raise ValueError(f"{type_.title()} name needs to be declared.")
-            if type_ == 'hook':
-                if name not in VALID_HOOKS_NAME:
-                    raise ValueError("Name of a hook doesn't match with expected value")
-                self.__hooks__.setdefault(name, [])
-                if self.__hooks__[name]:
-                    self.__hooks__[name] = [func]
-                else:
-                    self.__hooks__[name].append(func)
-                return func
-
-            if not is_dataclass(self):
-                raise TypeError("Dataclass is required for this class")
-
-            fields_ = tuple((field.name for field in fields(self)))
-            if name not in fields_:
-                raise ValueError("Expected validator to has name as column field")
-
-            fail = if_fail or f"{name} fails certain validator"
-
-            self.__validators__.setdefault(name, [])
-            validator = Validators(func, fail)
-            if self.__validators__[name]:
-                self.__validators__[name] = [validator]
-            else:
-                self.__validators__[name].append(validator)
-            return func
-        return function
+    @classmethod
+    def query(cls):
+        """Return Query Builder related to this model"""
+        return QueryBuilder(cls)
 
 def model(db: Database):
     """Initiate Model API compatible classes. Requires target to be a dataclass,
