@@ -5,9 +5,10 @@ from shlex import shlex
 from typing import Any, Iterable, Literal, Optional
 
 from .functions import ParsedFn, _function_extract
-from ._utils import check_one, null, check_iter
+from .errors import SecurityError
+from ._utils import check_one, null, check_iter, Null
 from .column import Column
-from .locals import _SQLITETYPES
+from .locals import _SQLITETYPES, SQLACTION
 from .signature import Signature
 from .typings import _MasterQuery, Data, Orders
 
@@ -108,8 +109,17 @@ def extract_signature(
         if val.normal_operator:
             string += f" {key}{middle}{val.value} and"
             last = 4
+        if val.is_in:
+            vals = tuple(
+                (f":prop_val_in{index}" for index, _ in enumerate(val.data)))  # type: ignore
+            string += f" {key} {middle} ({', '.join(vals)}) "
+            for key0, val0 in zip(vals, val.data):  # type: ignore
+                data[key0[1:]] = val0
+            continue
         if val.is_between:
             vdata: tuple[int, int] = val.data  # type: ignore
+            if not all(map(lambda x: isinstance(x, (int, float)), vdata)):
+                raise SecurityError("Values for between constraint is not int/float")
             string += f" {key} {middle} {vdata[0]!r} and {vdata[1]!r} and"
         if val.is_like:
             vdata: str = val.data  # type: ignore
@@ -181,7 +191,20 @@ function_extract = _function_extract
 
 
 def filter_extraction(string: str, shlexed: list[str]):
-    """A function step of table extraction. Used to replace quoted and parens with parameter."""
+    """
+    A function step of table extraction. Used to replace quoted and parens with parameter.
+
+    Args:
+        string (str): The input string containing the table creation SQL.
+        shlexed (list[str]): The tokenized list of the input string.
+
+    Returns:
+        tuple[dict[str, str], dict[str, str], str]: A tuple containing:
+            - A dictionary mapping placeholders to quoted strings.
+            - A dictionary mapping placeholders to parenthesized strings.
+            - The modified string with placeholders.
+    """
+
     quoted_wrap = {}
     paren_wrap = {}
     new_string = string
@@ -197,7 +220,6 @@ def filter_extraction(string: str, shlexed: list[str]):
         selected = new_string[index:last]
         new_string = new_string.replace(selected, f":wrap{len(paren_wrap)}")
         paren_wrap[f":wrap{len(paren_wrap)}"] = selected
-    # ? from now on, it's safe to use , again.
     return quoted_wrap, paren_wrap, new_string
 
 
@@ -288,6 +310,12 @@ def _iterate_etbc_step1(
     return string
 
 
+def _iterate_sql_action(action: SQLACTION):
+    if action == "null":
+        return "set null"
+    return action
+
+
 def extract_table_creations(
     columns: Iterable[Column], type_mappings: dict[str, str] | None = None
 ):
@@ -307,7 +335,7 @@ def extract_table_creations(
         stable, sname = column.source, column.source_column
 
         string += f" foreign key ({column.name}) references {stable} ({sname})\
- on delete {column.on_delete} on update {column.on_update},"
+ on delete {_iterate_sql_action(column.on_delete)} on update {_iterate_sql_action(column.on_update)},"  # type: ignore # pylint: disable=line-too-long
         # ! This might be a buggy code, i'm not sure yet.
     return string[1:-1]
 
@@ -332,13 +360,14 @@ def _setup_hashable(
     if isinstance(condition, list):
         cond = tuple(condition)
 
-    if isinstance(order, dict):
-        order_ = tuple(order.items())
+    if isinstance(order, tuple):
+        order_ = order
 
     if data:
         data_ = tuple(data.keys())
     return cond, order_, data_
 
+
 def _setup_limit_patch(table_name: str, condition: str, limit):
     check_one(table_name)
     if not isinstance(limit, int):
@@ -346,9 +375,28 @@ def _setup_limit_patch(table_name: str, condition: str, limit):
     return f"where rowid in (select rowid from {table_name}\
 {' '+condition if condition else ''} limit {limit})"
 
+
+def _parse_orders(order: CacheOrders):
+    if isinstance(order, tuple) and not isinstance(order[0], tuple):
+        ord_, order_by = order
+        return f"{ord_} {order_by}"
+    if isinstance(order, tuple) and isinstance(order[0], tuple):
+        return ", ".join(f"{ord_} {order_by}" for ord_, order_by in order)
+    raise TypeError("What?", type(order))
+
+def _remove_null(condition: dict[str, Any]) -> dict[str, Any]:
+    new = condition.copy()
+    for key, value in condition.items():
+        if value is Null:
+            del new[key]
+    if not new:
+        raise ValueError("After removing Null sentinel value, new data that would be inserted"
+                         "/updated returns empty dictionary.")
+    return new
+
 @lru_cache
-def _build_select(
-    table_name: str,  # pylint: disable=too-many-arguments
+def _build_select(  # pylint: disable=too-many-arguments
+    table_name: str,
     condition: CacheCond,
     only: OnlyColumn = None,  # type: ignore
     limit: int = 0,
@@ -363,19 +411,19 @@ def _build_select(
         only_, _ = only.parse_sql()
     elif isinstance(only, tuple):
         only_ = f"{', '.join(column_name for column_name in only)}"
-    elif only_ != '*' and isinstance(only_, str):
-        only_ = check_one(only) # type: ignore
+    elif only != "*" and isinstance(only, str):
+        only_ = check_one(only)  # type: ignore
 
-    query = f"select {only_} from {table_name}{' '+cond if cond else ''}"
+    query = f"select {only_} from {table_name}"
+    if cond:
+        query += f" {cond}"
+    if order and isinstance(order, tuple):
+        query += f" order by {_parse_orders(order)}"
     if limit:
         query += f" limit {limit}"
     if offset:
         query += f" offset {offset}"
-    if order:
-        query += " order by"
-        for ord_, order_by in order:
-            query += f" {ord_} {order_by},"
-        query = query[:-1]
+
     return query, data
 
 
@@ -392,13 +440,10 @@ def _build_update(
     new_str, updated = build_update_data(new_data)
     query = f"update {table_name} set {new_str} {cond}"
     if limit:
-        query = query.replace(cond, '')
+        query = query.replace(cond, "")
         query += _setup_limit_patch(table_name, cond, limit)
     if order:
-        query += " order by"
-        for ord_, order_by in order:
-            query += f" {ord_} {order_by},"
-        query = query[:-1]
+        query += f" order by {_parse_orders(order)}"
     # ? Require manual intervention to make sure updated is sync as
     # print(query)
     return query, data, updated
@@ -417,13 +462,10 @@ def _build_delete(
     cond, data = extract_signature(condition)
     query = f"delete from {table_name} {cond}"
     if limit:
-        query = query.replace(cond, '')
+        query = query.replace(cond, "")
         query += _setup_limit_patch(table_name, cond, limit)
     if order:
-        query += " order by"
-        for ord_, order_by in order:
-            query += f" {ord_} {order_by}"
-        query = query[:-1]
+        query += f" order by {_parse_orders(order)}"
     return query, data
 
 
@@ -436,8 +478,8 @@ values ({', '.join(val for val in converged.values())})"
     return query, data
 
 
-def build_select(
-    table_name: str,  # pylint: disable=too-many-arguments
+def build_select(  # pylint: disable=too-many-arguments
+    table_name: str,
     condition: Condition = None,
     only: tuple[str, ...] | ParsedFn | str | Literal["*"] = "*",
     limit: int = 0,
@@ -480,6 +522,7 @@ def build_update(
     Returns:
         tuple[str, dict[str, Any]]: query, query data
     """
+    new_data = _remove_null(new_data)
     cond, order_, ndata = _setup_hashable(condition, order, new_data)
     query, check, updated = _build_update(table_name, ndata, cond, limit, order_)
     return query, check | combine_keyvals(updated, new_data)
@@ -517,6 +560,7 @@ def build_insert(table_name: str, data: Data) -> tuple[str, dict[str, Any]]:
     Returns:
         tuple[str, dict[str, Any]]: query, query data
     """
+    data = _remove_null(data)
     _, _, ndata = _setup_hashable(None, None, data)
     query, _ = _build_insert(table_name, ndata)
     return query, data
